# This script generates the opcode.xml file
# by Andres Cabrera June 2006-2010
# Licensed under the GPL licence version 3 or later
# modification for empty arg in command and links on opcodes by Francois Pinot February 2007

from __future__ import print_function

from xml.dom import minidom
import os, glob

# categories holds the list of valid categories for opcodes
from categories import categories

XO = False
opcodelist = []

outfilename = 'opcodes.xml'
quickref = open(outfilename,'w')
quickref.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!-- "
               "Don't modify this file. It is generated automatically by opcodeparser.py\n"
               "This file is distributed under the GNU Free Documentation Licence-->")
quickref.write("<opcodes>\n")

entries = []
for i in categories:
  entries.append([])

manualfilename = 'manual.xml'
manual = open(manualfilename, 'r')
text = manual.read()
manual.close()

files = glob.glob('opcodes/*.xml')
files.extend(list(glob.glob('opcodes/*/*.xml')))
files.extend(list(glob.glob('vectorial/*.xml')))
files.extend(list(glob.glob('utility/*.xml')))
files.sort()

if files.index('opcodes/topXO.xml') >= 0:
    files.remove('opcodes/topXO.xml')

headerText = text[0:text.find('<book id="index"')]

special_entries = {
  'adds.xml': '<synopsis>a <opcodename>+</opcodename> b  (no rate restriction)</synopsis>\n',
  'dollar.xml': '<synopsis><opcodename>$NAME</opcodename></synopsis>\n<para/>',
  'divides.xml': '<synopsis>a <opcodename>/</opcodename> b  (no rate restriction)</synopsis>\n',
  'modulus.xml': '<synopsis>a <opcodename>%</opcodename> b  (no rate restriction)</synopsis>\n',
  'multiplies.xml': '<synopsis>a <opcodename>*</opcodename> b  (no rate restriction)</synopsis>\n',
  'opbitor.xml': '<synopsis>a <opcodename>'+'|</opcodename> b  (bitwise OR)</synopsis>\n',
  'opor.xml': '<synopsis>a <opcodename>'+'||</opcodename> b  (logical OR; not audio-rate)</synopsis>\n',
  'raises.xml': '<synopsis>a <opcodename>^</opcodename> b  (b not audio-rate)</synopsis>\n',
  'substracts.xml': '<synopsis>a <opcodename>-</opcodename> b (no rate restriction)</synopsis>\n',
  'ifdef.xml': ('<synopsis><opcodename>#ifdef</opcodename> NAME</synopsis><synopsis>  ....</synopsis>\n'
                '<synopsis><opcodename>#else</opcodename></synopsis><synopsis>  ....</synopsis>\n'
                '<synopsis><opcodename>#end</opcodename></synopsis>\n'),
  'define.xml': ('<synopsis><opcodename>#define</opcodename> NAME # replacement text #</synopsis>\n'
                 '<synopsis><opcodename>#define</opcodename> NAME(a&apos; b&apos; c&apos;) # replacement text #</synopsis>\n'),
  'include.xml': '<synopsis><opcodename>#include</opcodename> &quot;filename&quot;</synopsis>\n',
  'undef.xml': '<synopsis><opcodename>#undef</opcodename> NAME</synopsis>\n',
  '0dbfs.xml': '<synopsis><opcodename>0dbfs</opcodename> = iarg</synopsis>\n'
}

for i,filename in enumerate(files):
    source = open(filename, 'r')
    # Necessary to define entities>
    entryText = source.read().replace("\xef\xbb\xbf","")
    newfile = headerText + '<book id="index" lang="en">' + entryText + '</book>'
    newfile = newfile.replace("\r", "")
    source.close()
    xmldoc = minidom.parseString(newfile)
    xmldocId = xmldoc.documentElement.getAttribute('id')
    # Some files need special treatment (adds, dollar, divides, modulus, multiplies,
    # opbitor, opor, raises, subtracts, assign, ifdef, ifndef, define, include, undef)
    # There must be a better way to avoid loosing the entities when parsing the XML
    # file. Anyone???
    folder, base = os.path.split(filename)
    entry = special_entries.get(base)
    if entry is None:
        synopsis = xmldoc.getElementsByTagName('synopsis')
        entry = ''
        if synopsis:
            # There can be more than 1 synopsis per file
            for num in range(len(synopsis)):
                tmp = synopsis[num].toxml()
                if XO:
                    opcodename = tmp[tmp.find('<command>') + 9:tmp.find('</command>')]
                else:
                    opcodename = ""
                tmp = tmp.replace('<command>', '<opcodename>')
                entry += tmp.replace('</command>', '</opcodename>')

    # print "Entry ------ ", entry

    info = xmldoc.getElementsByTagName('refentryinfo')
    if info and entry:
        category = info[0].toxml()
        category = category[21:-23]
    else:
        print("no refentryinfo tag for file " + filename)
        category = "Miscellaneous"
        if entry:
            print(filename + " sent to Miscellaneous")
    desc = xmldoc.getElementsByTagName('refpurpose')
    description = ""
    if desc and entry:
        description = desc[0].firstChild.toxml().strip()
        # print(filename, category)
    else:
        print("no refpurpose tag for file " + filename)
    # print(category)
    match = False
    for j, thiscategory in enumerate(categories):
        if category == thiscategory:
            entries[j].append([entry, description])
            match = True
    if not match:
        print(filename + "---- WARNING! No Category Match!")

for i, category in enumerate(categories):
    if not category:
        print(f"No entries for category: {category} ...Skipping")
        continue
    quickref.write("<category name=\"" + categories[i] + "\">\n")
    count = 0
    for entry in entries[i]:
        entrydef, description = entry
        if not entrydef:
            continue
        # newentry = entry.pop(0)
        # entry = entry.replace("&dollar;", "$")
        # entry = entry.replace("&#160;", " ")
        quickref.write("<opcode>")
        if description:
            quickref.write("<desc>")
            quickref.write(description)
            quickref.write("</desc>")
        quickref.write(entrydef)
        quickref.write("</opcode>\n")
        count += 1
    quickref.write("</category>\n")
    print(str(count) + " entries in category: " + categories[i])

quickref.write('</opcodes>\n')
quickref.close()
print(entries)



# This python script generates the extra files needed for the frames
# version of the manual. It generates the files:
# indexframes.html, contents.html and title.html
# Output is based on html files created by Michael Rhoades
# Script written by Andres Cabrera June 2006
# Licensed under the GPL licence version 3 or later

import sys
from xml.dom import minidom

version = sys.argv[1:]
if (len(version)==0):
    version=['']
indexframesFile = open('html/indexframes.html','w')
creditsFile = open('html/credits.html','w')
titleFile = open('html/title.html','w')
contentsFile = open('html/contents.html','w')

indexFile = open('html/index.html','r')
index = indexFile.read()
indexFile.seek(0)
indexparse = minidom.parse(indexFile)

frameLink = '''
          <div>
            <h2 class="title"><a href="indexframes.html">Frames Version</a></h2>
          </div>'''
s = '''<div>
            <h1 class="title"><a id="index"></a>The Canonical Csound Reference Manual</h1>'''
titleEnd = index.find(s) + len(s) + 1
if (index[titleEnd:titleEnd+len(frameLink)]==frameLink):
    print '''**************************************
Warning: index.html already processed!
**************************************'''

# indexframes.html -----------------------------------------
indexframesFile.write('''<!-- generated by makeframes.py - DO NOT EDIT MANUALLY-->\n<html>\n<head>
<link rel="stylesheet" href="csound.css" type="text/css" />
<title>Csound ''' +version[0]+ ''' Manual - Frames Version</title>
</head>
<frameset cols="275,*">
  <frame name="left" scrolling="auto" noresize target="rtop" src="contents.html">
  <frameset rows="90,*">
    <frame name="rtop" target="rbottom" src="title.html" scrolling="no" marginheight="6">
    <frame name="rbottom" src="credits.html">
  </frameset>
  <noframes>
  <body>

  <p>This page uses frames, but your browser doesn\'t support them.</p>

  </body>
  </noframes>
</frameset>
</html>''')
indexframesFile.close()

# credits.html -----------------------------------------
creditsFile.write('''<?xml version="1.0" encoding="UTF-8" standalone="no"?>\n<!-- generated by makeframes.py -->
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
  <link rel="stylesheet" href="csound.css" type="text/css" />
  </head>
  <body>\n    ''')
creditsFile.write(index[index.find('<div class="book"'):index.find('Massachusetts Institute of Technology</p>')+106])
creditsFile.write('</div>\n  </body>\n</html>')  
creditsFile.close()

# title.html -----------------------------------------

titleFile.write('''<!-- generated by makeframes.py -->\n<html>\n<head>
  <title>The Canonical Csound Reference Manual</title>\n
    <link rel="stylesheet" href="csound.css" type="text/css" />
    <base target="rbottom">
    </head>
<body alink="#0000ff" link="#0000ff" vlink="#0000ff">
<p align="center"><font size="6">The Canonical Csound Reference Manual
</font>
<font size="4">
<br>
|
<a href="contents.html#a" style="text-decoration: none;">A</a>
|
<a href="contents.html#b" style="text-decoration: none;">B</a>
|
<a href="contents.html#c" style="text-decoration: none;">C</a>
|
<a href="contents.html#d" style="text-decoration: none;">D</a>
|
<a href="contents.html#e" style="text-decoration: none;">E</a>
|
<a href="contents.html#f" style="text-decoration: none;">F</a>
|
<a href="contents.html#g" style="text-decoration: none;">G</a>
|
<a href="contents.html#h" style="text-decoration: none;">H</a>
|
<a href="contents.html#i" style="text-decoration: none;">I</a>
|
<a href="contents.html#j" style="text-decoration: none;">J</a>
|
<a href="contents.html#k" style="text-decoration: none;">K</a>
|
<a href="contents.html#l" style="text-decoration: none;">L</a>
|
<a href="contents.html#m" style="text-decoration: none;">M</a>
|
<a href="contents.html#n" style="text-decoration: none;">N</a>
|
<a href="contents.html#o" style="text-decoration: none;">O</a>
|
<a href="contents.html#p" style="text-decoration: none;">P</a>
|
<a href="contents.html#r" style="text-decoration: none;">R</a>
|
<a href="contents.html#s" style="text-decoration: none;">S</a>
|
<a href="contents.html#t" style="text-decoration: none;">T</a>
|
<a href="contents.html#u" style="text-decoration: none;">U</a>
|
<a href="contents.html#v" style="text-decoration: none;">V</a>
|
<a href="contents.html#w" style="text-decoration: none;">W</a>
|
<a href="contents.html#x" style="text-decoration: none;">X</a>
|
<a href="contents.html#z" style="text-decoration: none;">Z</a>
|
<br>
<a href="ScoregensTop.html" target="rbottom" style="text-decoration: none;">Score Statements</a>
|
<a href="ScoreGenRef.html" target="rbottom" style="text-decoration: none;">GEN Routines</a>
|
<a href="MiscQuickref.html" target="rbottom" style="text-decoration: none;">Quick Reference</a> 
</font></p>\n</body>\n</html>''')
titleFile.close()

# contents.html -----------------------------------------

contentsFile.write('''<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!-- generated by makeframes.py -->
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
    <title>Contents</title>
    <link rel="stylesheet" href="csound.css" type="text/css" />
    <link rel="start" href="index.html" title="The Canonical Csound Reference Manual" />
    <link rel="next" href="PrefaceTop.html" title="Preface" />
    <base target="rbottom">
  </head>
  <body link="#0000FF" vlink="#0000FF" alink="#0000FF">
    <div class="navheader" style="width: 1137; height: 2">
      ''')

#routine to find the toc
navheader = indexparse.getElementsByTagName('div')
i = 0
while (navheader[i]!=''):
    #print navheader[i].attributes.keys()
    if (len(navheader[i].attributes.keys())!=0):
        if (navheader[i].attributes.keys()[0]=='class'):
            #print navheader[i].attributes['class'].value
            if(navheader[i].attributes['class'].value == 'toc'):
                break
    i += 1
    
toc = navheader[i]
sections = toc.getElementsByTagName('span')
a =  toc.getElementsByTagName('a')

for j in range(len(a)):
    if (a[j].attributes['href'].value=='OpcodesTop.html'):
        overview=a[j].parentNode.parentNode.nextSibling.nextSibling
    elif (a[j].attributes['href'].value=='ScoregensTop.html'):
        scoregens=a[j].parentNode.parentNode.nextSibling.nextSibling
        break


#print overview.toxml().encode('utf-8')

# Routine to make titles bigger and black, except Appendices

for j in range(len(sections)):
    #print "---------\n", sections[j].toxml()
    #raw_input()
    if ((sections[j].attributes['class'].value!='section')and
        (sections[j].attributes['class'].value!='chapter')and
        (sections[j].attributes['class'].value!='appendix')and
        (sections[j].attributes['class'].value!='refentrytitle')and
        (sections[j].attributes['class'].value!='refpurpose')and
        (sections[j].attributes['class'].value!='quote')and
        (sections[j].attributes['class'].value!='emphasis')):
        print "--------Title:", sections[j].childNodes[1].firstChild.toxml().encode('utf-8')
        font = minidom.Element('font')
        font.setAttribute('size','4')
        font.setAttribute('color','Black')
        font.appendChild(minidom.Element('br'))
        if (sections[j].childNodes[1].childNodes[0].toxml().encode('utf-8')=='Glossary'):
            font.appendChild(sections[j].childNodes[1].cloneNode(True))
        else:
            font.appendChild(sections[j].childNodes[1].childNodes[0].cloneNode(True))
        font.appendChild(minidom.Element('br'))
        #print sections[j].firstChild.toxml().encode('latin-1')
        #print font.toxml().encode('latin-1')
        parent = sections[j].parentNode
        parent.appendChild(font)
        parent.removeChild(sections[j])
        #spanElement = sections[j].getElementsByTagName('span')
        #try:
            #sections[j].removeChild(spanElement)
        #except:
            #print "No span for", sections[j].toxml()
        #print sections[j].toxml().encode('latin-1')
        #print font.toxml().encode('latin-1')
    elif (sections[j].attributes['class'].value=='appendix'):
        font = minidom.Element('font')
        font.setAttribute('size','4')
        font.appendChild(sections[j].childNodes[1].cloneNode(True))
        font.appendChild(minidom.Element('br'))
        #print sections[j].firstChild.toxml().encode('latin-1')
        #print font.toxml().encode('latin-1')
        parent = sections[j].parentNode
        parent.appendChild(font)
        parent.removeChild(sections[j])
        #print sections[j].toxml().encode('latin-1')
        #print font.toxml().encode('latin-1')
    else:
        #print sections[j].toxml().encode('latin-1')
        parent = sections[j].parentNode
        newNode = sections[j].getElementsByTagName('a')
        font = minidom.Element('font')
        font.setAttribute('size','2')
        #font.setAttribute('color','Black')
        #font.appendChild(minidom.Element('br'))
        #if (sections[j].attributes['class'].value!='refpurpose' and
          #sections[j].attributes['class'].value!='refentrytitle' and
          #sections[j].attributes['class'].value!='quote' and
          #sections[j].attributes['class'].value!='emphasis'):
        if newNode != []:
            #    font.appendChild(newNode[0].cloneNode(True))
            parent.appendChild(newNode[0].cloneNode(True))
#        parent.appendChild(font)
        parent.removeChild(sections[j])
        #print sections[j].toxml().encode('latin-1')
    

#print toc.toxml() #.encode('latin-1')

# routine to remove text description from opcode entries and
# add alphabetical references and letters
unwantedtext = overview.getElementsByTagName('dt')
alphabetcaps = []
alphabet = []

for num in range(65,93):
    alphabetcaps.extend(chr(num))
    alphabet.extend(chr(num+32))

count = -1
for text in unwantedtext:
    if count==15 or count==23:
        count = count + 1 #There is no 'Q' or 'Y'
    #print "text: ", text.toxml()
    text.removeChild(text.childNodes[0])
    #text.appendChild(minidom.Element('br'))
    #print "Unwanted: ", text.toxml()
    node = text.getElementsByTagName('a')
    if ((node[0].firstChild.toxml().encode('utf-8')[0]==alphabet[count+1])
        or(node[0].toxml().encode('utf-8')[0]==alphabetcaps[count+1])):
        count += 1
        string = '<b style="border:solid;"><a name="' + alphabet[count] + '">' + alphabetcaps[count] + '</a></b>'
        #print "string:", string
        letter = minidom.parseString(string)
        text.insertBefore(minidom.Element('br'), text.firstChild) 
        text.insertBefore(letter.firstChild, text.firstChild)
        print "unwanted2: ", text.firstChild
    #print text.toxml().encode('latin-1')


# routine to remove text description from score statement and GEN entries
unwantedtext = scoregens.getElementsByTagName('dt')
for text in unwantedtext:
    node = text.firstChild
    if node.nodeType == minidom.Node.ELEMENT_NODE and node.tagName == 'a':
        text.removeChild(text.childNodes[1])
        text.appendChild(minidom.Element('br'))
        

# Routines to remove indentation

#dt = toc.getElementsByTagName('dt')
#for j in range(len(dt)):
    #spanElement = dt.getElementsByTagName('span')
    #if spanElement.attributes['class'].value == 'section':
      #break
    #parent = dt[j].parentNode
    #print parent.toxml()
    #for k in range(len(dt[j].childNodes)):
        #parent.insertBefore(dt[j].firstChild ,dt[j])
    #parent.removeChild(dt[j])


#dl = toc.getElementsByTagName('dl')
#for j in range(len(dl)):
    #parent = dl[j].parentNode
    #for k in range(len(dl[j].childNodes)):
        #parent.insertBefore(dl[j].firstChild ,dl[j])
    #parent.removeChild(dl[j])
    
#dd = toc.getElementsByTagName('dd')
#for j in range(len(dd)):
    #parent = dd[j].parentNode
    #if (parent.nodeName!='dd'):
        #for k in range(len(dd[j].childNodes)):
            #parent.insertBefore(dd[j].firstChild ,dd[j])
        #parent.removeChild(dd[j])



#print toc
#print toc.toxml().encode('latin-1')
toc = toc.toxml().encode('utf-8')
contentsFile.write(toc)
contentsFile.close()

# Modify index.html to include a link to the frames version

if (index[titleEnd:titleEnd+len(frameLink)]!=frameLink):
    newindex = index[0:titleEnd]+ frameLink
    newindex += index[titleEnd:]
    indexFile.close()
    indexFile = open('html/index.html','w')
    indexFile.seek(0)
    indexFile.write(newindex)
    indexFile.close()





# -*- coding: utf-8 -*-
# This script generates the misc/quickref.html file
# by Andres Cabrera June 2006
# Licensed under the GPL licence version 3 or later
# modification for empty arg in command and links on opcodes by Francois Pinot February 2007

from __future__ import print_function
from xml.dom import minidom
import os, glob, sys

# categories holds the list of valid categories for opcodes
from categories import categories

XO = False
opcodelist = []

if sys.argv.count("-x"):
  XO = True
  file = open("opcode_listXO.txt", 'r')
  removedopcodes = file.read().split()
  file.close()

entries = []
example_cats = []
for i in categories:
  entries.append([])
  example_cats.append([])

# Generate a file listing all examples

outfilename = 'misc/examples.xml'
examples = open(outfilename,'w')
examples.write("\n<!-- Don't modify this file. It is generated automatically by quickref.py-->\n")
examples.write('''<appendix id="MiscExamples">
  <title>List of examples</title>''')

files = glob.glob('opcodes/*.xml')
try:
    files.remove('opcodes/template.xml')
except:
    pass
files.sort()
for i,filename in enumerate(files):
    entry = ''
    #print(filename)
    source = open(filename, 'r')
    text = source.read()
    source.close()
    cat_text = text[text.find("<refentryinfo><title>") + 21: text.find("</title></refentryinfo>")]
    if categories.count(cat_text) > 0:
      cat = categories.index(cat_text);
      #print(cat_text, cat)
      while text.find("examples-xml/") != -1:
        file_to_add = text[text.find("examples-xml/"):text.find(".csd",text.find("examples-xml/") )+ 4]
        if len(file_to_add) < 45: #yes, very flaky, I know...
          entries[cat].append(file_to_add)
        else:
          print(file_to_add)
        text = text[text.find("examples-xml/") + 10:]

for i,catname in enumerate(categories):
    print(catname)
    if len(entries[i]) > 0:
        title ="<formalpara><title>"
        title += catname
        #title += "</title><simpara />\n"
        title += "</title><simpara />\n"
        examples.write(title)
        for j,ex in enumerate(entries[i]):
            if ex != "":
                line = '<para><ulink url="'
                line += ex.replace("-xml", '') + '"><citetitle>' + ex.replace("examples-xml/",'')
                print(line)
                #line += "</citetitle></ulink></para><simpara />\n"
                line += "</citetitle></ulink></para><simpara />\n"
                examples.write(line)
        examples.write("</formalpara>\n")
  
examples.write('</appendix>\n')
examples.close()


#Now generate the quickref file

#Reset entries
entries = []
for i in categories:
    entries.append([])

outfilename = 'misc/quickref.xml' if not XO else 'misc/quickrefXO.xml'
quickref = open(outfilename,'w')
quickref.write("\n<!-- Don't modify this file. It is generated automatically by quickref.py-->\n")
quickref.write('''<part>
  <title>Opcode Quick Reference</title>
  <chapter id="MiscQuickref">
  <title>Opcode Quick Reference</title>''')

manualfilename = 'manual.xml' if not XO else 'manualXO.xml'
manual = open(manualfilename, 'r')
text = manual.read()
manual.close()

files = glob.glob('opcodes/*.xml')
try:
    files.remove('opcodes/template.xml')
except:
    pass
files[len(files):]=glob.glob('opcodes/*/*.xml')
files[len(files):]=glob.glob('vectorial/*.xml')
files[len(files):]=glob.glob('utility/*.xml')
try:
    files.remove('utility/mkdb.xml')
except:
    pass
files.sort()

headerText = text[0:text.find('<book id="index"')]

for i,filename in enumerate(files):
    entry = ''
    #print(file)
    source = open(filename, 'r')
    newfile = source.read()
    source.close()

    refStart = newfile.find("<refentry")

    if(refStart < 0):
        continue
    elif(refStart > 0):
#        print('Trimming file: ', filename, ' ', refStart)
        newfile = newfile[refStart:]
    
    # Necessary to define entities
    newfile = headerText + newfile

    #print(text)
    try:
        xmldoc = minidom.parseString(newfile)
    except:
        print('>>> Failed to parse:', filename)
        continue
    xmldocId = xmldoc.documentElement.getAttribute('id')
    # Some files need special treatment (adds, dollar, divides, modulus, multiplies,
    # opbitor, opor, raises, subtracts, assign, ifdef, ifndef, define, include, undef)
    # There must be a better way to avoid loosing the entities when parsing the XML
    # file. Anyone???
    if (filename == 'opcodes' + os.sep + 'adds.xml'):
        entry = '<synopsis>a <link linkend="'+xmldocId+'">'+'&plus;</link> b  (no rate restriction)</synopsis>\n<para/>'
    elif (filename == 'opcodes' + os.sep + 'dollar.xml'):
        entry = '<synopsis><link linkend="'+xmldocId+'">'+'&dollar;NAME</link>&#160;</synopsis>\n<para/>'
    elif (filename == 'opcodes' + os.sep + 'divides.xml'):
        entry = '<synopsis>a <link linkend="'+xmldocId+'">'+'&sol;</link> b  (no rate restriction)</synopsis>\n<para/>'
    elif (filename == 'opcodes' + os.sep + 'modulus.xml'):
        entry = '<synopsis>a <link linkend="'+xmldocId+'">'+'&percnt;</link> b  (no rate restriction)</synopsis>\n<para/>'
    elif (filename == 'opcodes' + os.sep + 'multiplies.xml'):
        entry = '<synopsis>a <link linkend="'+xmldocId+'">'+'&ast;</link> b  (no rate restriction)</synopsis>\n<para/>'
    elif (filename == 'opcodes' + os.sep + 'opbitor.xml'):
        entry = '<synopsis>a <link linkend="'+xmldocId+'">'+'&verbar;</link> b  (bitwise OR)</synopsis>\n<para/>'
    elif (filename == 'opcodes' + os.sep + 'opor.xml'):
        entry = '<synopsis>a <link linkend="'+xmldocId+'">'+'&verbar;&verbar;</link> b  (logical OR; not audio-rate)</synopsis>\n<para/>'
    elif (filename == 'opcodes' + os.sep + 'raises.xml'):
        entry = '<synopsis>a <link linkend="'+xmldocId+'">'+'&circ;</link> b  (b not audio-rate)</synopsis>\n<para/>'
    elif (filename == 'opcodes' + os.sep + 'subtracts.xml'):
        entry = '<synopsis>a <link linkend="'+xmldocId+'">'+'&minus;</link> b (no rate restriction)</synopsis>\n<para/>'
#    elif (filename == 'opcodes' + os.sep + 'assign.xml'):
#        entry = '''<synopsis> <link linkend="'+xmldocId+'">'+'&minus;</link>a  (no rate restriction)</synopsis>
#        <synopsis> <link linkend="'+xmldocId+'">'+'&plus;</link>a  (no rate restriction)</synopsis>\n'''
    elif (filename == 'opcodes' + os.sep + 'ifdef.xml'):
          entry = '<synopsis><link linkend="'+xmldocId+'">'+'&num;ifdef</link> NAME</synopsis><synopsis>  ....</synopsis>' + \
                  '<synopsis><link linkend="'+xmldocId+'">'+'&num;else</link>&#160;</synopsis><synopsis>  ....</synopsis>' + \
                  '<synopsis><link linkend="'+xmldocId+'">'+'&num;end</link>&#160;</synopsis>\n<para/>'
    elif (filename == 'opcodes' + os.sep + 'ifndef.xml'):
          entry='<synopsis><link linkend="'+xmldocId+'">'+'&num;ifndef</link> NAME</synopsis><synopsis>  ....</synopsis>' + \
                '<synopsis><link linkend="'+xmldocId+'">'+'&num;else</link>&#160;</synopsis><synopsis>  ....</synopsis>' + \
                '<synopsis><link linkend="'+xmldocId+'">'+'&num;end</link>&#160;</synopsis>\n<para/>'
    elif (filename == 'opcodes' + os.sep + 'define.xml'):
          entry='<synopsis><link linkend="'+xmldocId+'">'+'&num;define</link> NAME &num; replacement text &num;</synopsis><para/>\n' + \
    '<synopsis><link linkend="'+xmldocId+'">'+'&num;define</link> NAME(a&apos; b&apos; c&apos;) &num; replacement text &num;</synopsis>\n<para/>'
    elif (filename == 'opcodes' + os.sep + 'include.xml'):
          entry='<synopsis><link linkend="'+xmldocId+'">'+'&num;include</link> &quot;filename&quot;</synopsis>\n<para/>'
    elif (filename == 'opcodes' + os.sep + 'undef.xml'):
          entry='<synopsis><link linkend="'+xmldocId+'">'+'&num;undef</link> NAME</synopsis>\n<para/>'
    else:
        synopsis = xmldoc.getElementsByTagName('synopsis')
        if (len(synopsis) != 0):
            # There can be more than 1 synopsis per file
            for num in range(len(synopsis)):
                tmp = synopsis[num].toxml()
                if XO:
                    opcodename = tmp[tmp.find('<command>') + 9:tmp.find('</command>')]
                else:
                    opcodename = ""
                if XO and removedopcodes.count(opcodename) == 0:
                    print("Removed ----------------------", opcodename)
                else:
                    if tmp[-21:] == "</command></synopsis>":    # no arg, insert nbsp
                        tmp = tmp[:-11] + "&#160;</synopsis>"
                    tmp = tmp.replace('<command>', '<link linkend="' + xmldocId + '">')
                    entry += tmp.replace('</command>', '</link>')
            if entry != '':
                entry += '<para/>'
        else:
            #print("no synopsis tag for file: " + file)
            entry = ''
    #print("Entry ------ ", entry)

    info = xmldoc.getElementsByTagName('refentryinfo')
    if (len(info)!=0 and entry != ''):
        category = info[0].toxml()
        category = category[21:-23]
        print(category)
    else:
        print("no refentryinfo tag for file " + filename)
        category = "Miscellaneous"
        if (entry!=''):
            print(filename + " sent to Miscellaneous")
    #print(category)
    match = False
    for j, thiscategory in enumerate(categories):
        if (category == thiscategory):
            entries[j].append(entry+ '\n')
            match = True
    if match == False:
        print(filename + "WARNING! No Category Match!")

for i in range(len(categories)):
    if (len(entries[i])==0):
        print("No entries for category: "+categories[i]+"...Skipping")
        continue
    quickref.write("<para></para><formalpara>\n")
    quickref.write("<title>"+ categories[i] + "</title>\n<para>\n<para/>\n")
    count = 0
    for j in range(len(entries[i])):
        quickref.write(entries[i].pop(0)) # + '\n')
        count += 1
    quickref.write("</para></formalpara>\n<para></para>")
    print(str(count) + " entries in category: " + categories[i])

quickref.write('</chapter></part>\n')
quickref.close()
print(entries)



#categories and their order are defined here:
categories = ['Orchestra Syntax:Header',
              'Orchestra Syntax:Block Statements',
              'Orchestra Syntax:Macros',
              'Signal Generators:Additive Synthesis/Resynthesis',
              'Signal Generators:Basic Oscillators',
              'Signal Generators:Dynamic Spectrum Oscillators',
              'Signal Generators:FM Synthesis',
              'Signal Generators:Granular Synthesis',
              'Signal Generators:Hyper Vectorial Synthesis',
              'Signal Generators:Linear and Exponential Generators',
              'Signal Generators:Envelope Generators',
              'Signal Generators:Models and Emulations',
              'Signal Generators:Phasors',
              'Signal Generators:Random (Noise) Generators',
              'Signal Generators:Sample Playback',
              'Signal Generators:Scanned Synthesis',
              'Signal Generators:STK Opcodes',
              'Signal Generators:Table Access',
              'Signal Generators:Wave Terrain Synthesis',
              'Signal Generators:Waveguide Physical Modeling',
              'Signal I/O:File I/O',
              'Signal I/O:Signal Input',
              'Signal I/O:Signal Output',
              'Signal I/O:Software Bus',
              'Signal I/O:Printing and Display',
              'Signal I/O:Soundfile Queries',
              'Signal Modifiers:Amplitude Modifiers',
              'Signal Modifiers:Convolution and Morphing',
              'Signal Modifiers:Delay',
              'Signal Modifiers:Panning and Spatialization',
              'Signal Modifiers:Reverberation',
              'Signal Modifiers:Sample Level Operators',
              'Signal Modifiers:Signal Limiters',
              'Signal Modifiers:Special Effects',
              'Signal Modifiers:Standard Filters',
              'Signal Modifiers:Standard Filters:Resonant',
              'Signal Modifiers:Standard Filters:Control',
              'Signal Modifiers:Specialized Filters',
              'Signal Modifiers:Waveguides',
              'Signal Modifiers:Waveshaping',
              'Signal Modifiers:Comparators and Accumulators',
              'Instrument Control:Clock Control',
              'Instrument Control:Conditional Values',
              'Instrument Control:Compilation',
              'Instrument Control:Duration Control',
              'Instrument Control:Invocation',
              'Instrument Control:Program Flow Control',
              'Instrument Control:Realtime Performance Control',
              'Instrument Control:Initialization and Reinitialization',
              'Instrument Control:Sensing and Control',
              'Instrument Control:Stacks',
              'Instrument Control:Subinstrument Control',
              'Instrument Control:Time Reading',
              'Jacko Opcodes',
              'Lua Opcodes',
              'Serial I/O',
              'Table Control',
              'Table Control:Table Queries',
              'Table Control:Dynamic Selection',
              'Table Control:Read/Write Operations',
              'FLTK:Containers',
              'FLTK:Valuators',
              'FLTK:Other',
              'FLTK:Appearance',
              'Mathematical Operations:Arithmetic and Logic Operations',
              'Mathematical Operations:Arrays',
              'Mathematical Operations:Comparators and Accumulators',
              'Mathematical Operations:Mathematical Functions',
              'Mathematical Operations:Trigonometric Functions',
              'Mathematical Operations:Amplitude Functions',
              'Mathematical Operations:Random Functions',
              'Mathematical Operations:Opcode Equivalents of Functions',
              'Mathematical Operations:Vectors',
              'Pitch Converters:Functions',
              'Pitch Converters:Tuning Opcodes',
              'Real-time MIDI:Input',
              'Real-time MIDI:Output',
              'Real-time MIDI:Converters',
              'Real-time MIDI:Generic I/O',
              'Real-time MIDI:Event Extenders',
              'Real-time MIDI:Note Output',
              'Real-time MIDI:MIDI/Score Interoperability',
              'Real-time MIDI:System Realtime',
              'Real-time MIDI:Slider Banks',
              'Signal Flow Graph Opcodes',
              'Spectral Processing:STFT',
              'Spectral Processing:LPC',
              'Spectral Processing:Non-Standard',
              'Spectral Processing:Streaming',
              'Spectral Processing:ATS',
              'Spectral Processing:Loris',
              'Spectral Processing:Other',
              'Strings:Definition',
              'Strings:Manipulation',
              'Strings:Conversion',
              'Vectorial:Tables',
              'Vectorial:Scalar operations',
              'Vectorial:Vectorial operations',
              'Vectorial:Envelopes',
              'Vectorial:Limiting and Wrapping',
              'Vectorial:Delay Paths',
              'Vectorial:Random',
              'Vectorial:Cellular Automata',
              'Zak Patch System',
              'Plugin Hosting:DSSI and LADSPA',
              'OSC',
              'Faust Opcodes',
              'Network',
              'Remote Opcodes',
              'Mixer Opcodes',
              'Ableton Link Opcodes',
              'Python Opcodes',
              'Image Processing Opcodes',
              'Array Opcodes',
              'Array Operations: Fast Fourier Transform',
              'Array Operations: complex numbers',
              'Array Operations: Discrete Cosine Transform',
              'Array Operations: dot product',
              'Array Operations: Mel scale filterbank',
              'Array Operations: sorting',
              'Array Operations: Cepstrum',
              'Deprecated',
              'Miscellaneous',
              'Testing',
              'Utilities']



# -*- coding: utf-8 -*-
#
# Copyright (C) 2007 François Pinot
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

from __future__ import print_function
import glob, re
from pygments import highlight
from pygments.formatter import Formatter
from pygments.lexers import CsoundDocumentLexer, CsoundOrchestraLexer, CsoundScoreLexer, _csound_builtins
from pygments.token import Token, STANDARD_TYPES
from xml.sax.saxutils import escape


# FluidSynth opcodes are not installed with Csound 6.11 and later on macOS, and
# they were removed from _csound_builtins.OPCODES at
# https://bitbucket.org/birkenfeld/pygments-main/diff/pygments/lexers/_csound_builtins.py?diff2=1d8eed62b214
opcodeNames = '''
fluidAllOut
fluidCCi
fluidCCk
fluidControl
fluidEngine
fluidInfo
fluidLoad
fluidNote
fluidOut
fluidProgramSelect
fluidSetInterpMethod
'''.split()
for opcodeName in opcodeNames:
    if opcodeName in _csound_builtins.OPCODES:
        print('_csound_builtins.OPCODES already contains ‘' + opcodeName + '’')
    else:
        _csound_builtins.OPCODES.add(opcodeName)


# To match the syntax highlighting from
# https://github.com/csound/manual/tree/c1b097bae66e04c2b11395f12a03f0d67fc1f059
# as closely as possible, change the token type of score statements from Keyword
# to Name.Builtin so that they look like opcodes. With Pygments 2.3 and later,
# this doesn’t apply to m statements (https://csound.com/docs/manual/m.html),
# n statements (https://csound.com/docs/manual/n.html), or p symbols
# (https://csound.com/docs/manual/ScoreNextp.html); see
# https://bitbucket.org/birkenfeld/pygments-main/commits/b60b37f36a03802e0c5ed412268d37bbc55eb9fb#Lpygments/lexers/csound.pyT159
stateTuple = CsoundScoreLexer.tokens['root'][3]
CsoundScoreLexer.tokens['root'][3] = stateTuple[:1] + (Token.Name.Builtin,) + stateTuple[2:]


# See http://pygments.org/docs/formatterdevelopment/.
class DocBookFormatter(Formatter):
    def format(self, tokensource, outfile):
        outfile.write('<programlisting>\n')

        currentTypeString = ''
        currentValue = ''
        for ttype, value in tokensource:
            typeString = STANDARD_TYPES[ttype]
            if typeString == currentTypeString:
                currentValue += value
            else:
                if currentTypeString == 'k' and re.match('end(?:in|op)|instr|opcode', currentValue):
                    # If the current token is a Keyword and is one of endin,
                    # endop, instr, or opcode, treat it as a Keyword.Declaration
                    # token. This is only necessary with Pygments 2.1 through
                    # 2.2. In Pygments 2.3 and later, endin, endop, instr, and
                    # opcode are Keyword.Declaration tokens; see
                    # https://bitbucket.org/birkenfeld/pygments-main/commits/b60b37f36a03802e0c5ed412268d37bbc55eb9fb#Lpygments/lexers/csound.pyT249
                    currentTypeString = 'kd'
                elif currentTypeString == 'kt' and re.match('g?[afikSw]', currentValue) and typeString == 'n':
                    # If the current token is a Keyword.Type, has a value that
                    # matches a Csound type sigil, and is followed immediately
                    # by a name, then it’s a type sigil. Treat it as a Text
                    # token.
                    currentTypeString = ''
                self.writeToken(currentTypeString, currentValue, outfile)
                currentTypeString = typeString
                currentValue = value

        self.writeToken(currentTypeString, currentValue, outfile)

        outfile.write('</programlisting>\n')

    def writeToken(self, typeString, value, outfile):
        escapedValue = escape(value)
        # Don’t wrap Text tokens in emphasis elements.
        if typeString == '':
            outfile.write(escapedValue)
        else:
            outfile.write('<emphasis role="' + typeString + '">' + escapedValue + '</emphasis>')


for path in glob.glob('examples/*.csd'):
    with open(path, 'r') as file:
        code = file.read()
        with open(path.replace('examples', 'examples-xml') + '.xml', 'w') as file2:
            file2.write('<refsect1>\n')
            file2.write(highlight(code, CsoundDocumentLexer(), DocBookFormatter()))
            file2.write('</refsect1>\n')

path = 'examples/table1.inc'
with open(path, 'r') as file:
    code = file.read()
with open(path.replace('examples/', 'examples-xml/') + '.xml', 'w') as file:
    file.write(highlight(code, CsoundOrchestraLexer(), DocBookFormatter()))
